@@ -1,15 +1,10 @@
+use crate::traits::Protectable;
+
 use std::default::Default;
 use std::fmt;
-use std::io::Error;
-use std::ops::{Deref, DerefMut, Drop};
+use std::ops::{Deref, DerefMut, Index, IndexMut, Drop};
 use std::sync::atomic::{AtomicUsize, Ordering};
 
-pub trait Protectable {
-	fn lock(&self) -> Result<(), Error>;
-	fn unlock(&self) -> Result<(), Error>;
-	fn unlock_mut(&mut self) -> Result<(), Error>;
-}
-
 pub struct Guard<T: Protectable>(AtomicUsize, T);
 pub struct Ref<'t, T: Protectable>(pub &'t Guard<T>);
 pub struct RefMut<'t, T: Protectable>(pub &'t mut Guard<T>);
@@ -159,6 +154,14 @@ impl<T: Protectable + Deref> Deref for Ref<'_, T> {
 	}
 }
 
+impl<T: Protectable + Index<I>, I> Index<I> for Ref<'_, T> {
+	type Output = T::Output;
+
+	fn index(&self, index: I) -> &Self::Output {
+		&self.0.1[index]
+	}
+}
+
 impl<T: Protectable> Drop for Ref<'_, T> {
 	fn drop(&mut self) {
 		self.0.release();
@@ -189,6 +192,20 @@ impl<T: Protectable + Deref> Deref for RefMut<'_, T> {
 	}
 }
 
+impl<T: Protectable + Index<I>, I> Index<I> for RefMut<'_, T> {
+	type Output = T::Output;
+
+	fn index(&self, index: I) -> &Self::Output {
+		&self.0.1[index]
+	}
+}
+
+impl<T: Protectable + Index<I> + IndexMut<I>, I> IndexMut<I> for RefMut<'_, T> {
+	fn index_mut(&mut self, index: I) -> &mut Self::Output {
+		&mut self.0.1[index]
+	}
+}
+
 impl<T: Protectable + Deref + DerefMut> DerefMut for RefMut<'_, T> {
 	fn deref_mut(&mut self) -> &mut Self::Target {
 		&mut *self.0.1
